import { HttpCRUDTranslator } from '@/http-crud-translator/http-crud-translator';
import { Utilities } from '@/util/Utilities';
import {
    Container,
    FileInput,
    Form,
    Frame,
    LoadingService,
    Select, SelectMultiple,
} from '@zeedhi/common';
import { IDictionary, IEventParam, Metadata } from '@zeedhi/core';
import { IterableCarousel } from '@zeedhi/teknisa-components-common';
import FileUpload from '../FileUpload';

export class Fm00001CriarOrdensServicoController {
    private carouselOrdemVisivel: boolean = false;

    public changeTipoDeLocal({ component }: IEventParam<Select>) {
        const tipoLocal = Metadata.getInstance<Select>('selectTipoLocalOrdemServico');
        const local = Metadata.getInstance<Select>('select-local-ordem-servico');

        tipoLocal.value = null;
        local.value = null;

        if (component.value) {
            tipoLocal.disabled = false;
            tipoLocal.datasource.setFilter({
                CDCLIENTE: component.value,
            });
        } else {
            tipoLocal.disabled = true;
            local.disabled = true;
        }
    }

    public changeLocal({ component }: IEventParam<Select>) {
        const local = Metadata.getInstance<Select>('select-local-ordem-servico');

        if (component.value) {
            local.disabled = false;

            local.datasource.setFilter({
                NRTIPOLOCALCLI: component.value,
                UNICA: 'S',
                CDCLIENTE: Metadata.getInstance<Select>('selectClienteOrdemServico').value,
            });
        } else {
            local.value = null;
            local.disabled = true;
        }
    }

    public changeResponsavel({ component }: IEventParam<Select>) {
        const elements = {
            operador: Metadata.getInstance<SelectMultiple>('operador-ordem-servico'),
            equipe: Metadata.getInstance<Select>('equipe-ordem-servico'),
            profissionalE: Metadata.getInstance<Select>('profissional-externo-ordem-servico'),
            local: Metadata.getInstance<Select>('select-local-ordem-servico'),
        };

        // Resetar todos os elementos para estado inicial
        const resetAll = () => {
            this.setElementState(elements.operador, false);
            this.setElementState(elements.equipe, false);
            this.setElementState(elements.profissionalE, false);
        };

        // Aplicar lógica com base no valor do componente
        resetAll();
        switch (component.value) {
            case '1':
                this.setElementState(elements.operador, true, undefined, { type: 'required', params: {} });
                elements.operador.datasource.setFilter({ NRLOCALCLIENTE: elements.local.value });
                break;
            case '2':
                this.setElementState(elements.equipe, true, undefined, { type: 'required', params: {} });
                elements.equipe.datasource.setFilter({ NRLOCALCLIENTE: elements.local.value });
                break;
            case '3':
                this.setElementState(elements.profissionalE, true, undefined, { type: 'required', params: {} });
                break;
            default:
                // Default logic handled by resetAll
                break;
        }
    }

    public setElementState(element: any, isVisible: boolean, value: any = null, validation: { type: string; params: any } | null = null) {
        element.isVisible = isVisible;
        if (value !== undefined) {
            element.value = value;
        }
        if (validation) {
            element.addValidation(validation.type, validation.params);
        } else {
            element.removeValidation('required');
        }
    }

    public mostrarFotoOrdem({ component }: any) {
        const carouselFotos = Metadata.getInstance<IterableCarousel>('carouselFotosOrdem');
        carouselFotos.iterable.datasource.updateData([]);
        const imagens: any = [];

        component.value.forEach((element: any, pos: any) => {
            imagens.push({
                id: `numero_${pos}`,
                src: window.URL.createObjectURL(element),
            });
        });
        carouselFotos.iterable.datasource.updateData(imagens);
        this.carouselOrdemVisivel = component.value.length > 0;
    }

    public async criarOrdemServico() {
        const form = Metadata.getInstance<Form>('form-criar-ordem-servico');
        const imgs = await this.updloadPictures('file-input-fotos-ordem');
        const profissionalE = Metadata.getInstance<Select>('profissional-externo-ordem-servico');

        if (Utilities.validateForm(form)) {
            LoadingService.show();
            HttpCRUDTranslator.post('/criarOrdemServico', {
                NMORDEMOSERV: form.value.inputNomeOrdem,
                IDSITUACAO: form.value.selectStatusOrdemServico,
                NREQUIPAMENTO: form.value.selectEquipamentoOrdemServico,
                NRCATEGORIAMANUT: form.value.selectCategoriaManutOrdemServico,
                NRTIPOMANUTENCAO: form.value.selectTipoManutOrdemServico,
                NRPRIORIDADEMANUT: form.value.selectPrioridadeOrdemServico,
                DTORDEMSERV: form.value.inputDataOrdemServico,
                DSORDEMSERV: form.value.inputDescricaoOrdemServico,
                NRESCOPOS: form.value.selectEscopoOrdemServico,
                FOTOS: imgs,
                NRPROFISSIONALEXTERNO: profissionalE.value,
                NREQUIPE: Metadata.getInstance<Select>('equipe-ordem-servico').value,
                CDOPERADOR: Metadata.getInstance<SelectMultiple>('operador-ordem-servico').value,
                NRLOCALCLIENTE: Metadata.getInstance<Select>('select-local-ordem-servico').value,
                NREPI: Metadata.getInstance<Select>('select-epis-ordem-servico').value,
                NRMODELOPOP: Metadata.getInstance<Select>('select-modelo-pop-ordem-servico').value,
            }).then(() => {
                this.limparCampos({ component: form });
                Utilities.showSuccessMessage();
            }).catch((error: any) => {
                Utilities.showErrorMessage(error);
            }).finally(() => {
                LoadingService.hide();
            });
        }
    }

    public async updloadPictures(name: string) {
        const fileInputObject = Metadata.getInstance<FileInput>(name).value;
        let arquivos: any = [];
        if (fileInputObject && fileInputObject.length > 0) {
            const types: IDictionary = ['image/jpg', 'image/jpeg', 'image/png'];
            const buildFilesPromises = fileInputObject.map((file: any) => {
                if (types.includes(file.type)) {
                    return FileUpload.buildFileSendObject(file);
                }
                return [];
            });
            await Promise.all(buildFilesPromises).then((files: any) => {
                arquivos = files;
            });
        }
        return arquivos;
    }

    public limparCampos({ component }: IEventParam<Form>) {
        const fileInput = Metadata.getInstance<FileInput>('file-input-fotos-ordem');
        const profissionalE = Metadata.getInstance<Select>('profissional-externo-ordem-servico');
        const equipe = Metadata.getInstance<Select>('equipe-ordem-servico');
        const operador = Metadata.getInstance<SelectMultiple>('operador-ordem-servico');
        const localCliente = Metadata.getInstance<Select>('select-local-ordem-servico');
        const EPIs = Metadata.getInstance<Select>('select-epis-ordem-servico');
        const modeloPop = Metadata.getInstance<Select>('select-modelo-pop-ordem-servico');
        const carouselFotos = Metadata.getInstance<IterableCarousel>('carouselFotosOrdem');

        fileInput.value = '';
        profissionalE.setValue([]);
        equipe.setValue([]);
        operador.setValue([]);
        localCliente.setValue([]);
        EPIs.setValue([]);
        modeloPop.setValue([]);

        component.clearForm();

        carouselFotos.iterable.datasource.updateData([]);
        this.carouselOrdemVisivel = false;
        this.fecharPaginaCriarOrdem();
    }

    public fecharPaginaCriarOrdem() {
        const frameExec = Metadata.getInstance<Frame>('frame-exec-ordem');
        const frameRecuas = Metadata.getInstance<Frame>('frame-recusar_ordem');
        const frameCriarOrdem = Metadata.getInstance<Frame>('frame-criar-ordem');
        const paginaPrincipal = Metadata.getInstance<Container>('container-interno-ordem-servico');

        frameExec.isVisible = false;
        frameRecuas.isVisible = false;
        frameCriarOrdem.isVisible = false;
        paginaPrincipal.isVisible = true;
    }
}
